package rushhour.abstraction;

import java.util.List;

public interface IVehicule {

	public static final boolean HORIZONTALE = false;
	public static final boolean VERTICALE   = true;
	
	
	/**
	 *  Retourne le nombre de cases occupees par le vehicule
	 * @return le nombre de cases occupees par le vehicule
	 */
	public int getLongueur();
	
	/**
	 * Retourne l'orientation du vehicule (IVehicule.HORIZONTALE ou IVehicule.VERTICALE)
	 * @return l'orientation du vehicule (IVehicule.HORIZONTALE ou IVehicule.VERTICALE)
	 */
	public boolean getOrientation();
	
	/**
	 * Retourne la teinte du vehicule
	 * @return la teinte du vehicule
	 */
	public int getCouleur();
	
	/**
	 * Retourne l'inscription du vehicule
	 * @return l'inscription du vehicule
	 */
	public String getInscription();
	
	/**
	 * Modifie le vehicule de sorte que la case la plus en haut a gauche qu'il occupe soit c
	 * Les autres caracteristiques (orientation, longueur, couleur et description) ne sont pas modifiees.
	 * @param c, une case de l'aire de jeu
	 */
	public void deplacerEn(Case c);
	
	/**
	 * Retourne la liste des cases occupees par le vehicule en allant de la case la plus en haut a gauche occupee a la case la plus en bas a droite occupee. 
	 * @return la liste des cases occupees par le vehicule en allant de la case la plus en haut a gauche occupee a la case la plus en bas a droite occupee.
	 */
	public List<Case> getCases() ;
	
	/**
	 * Retourne true si le vehicule occupe (entre autres) la case c.
	 * @param c, une case de l'aire de jeu
	 * @return true si le vehicule occupe (entre autres) la case c.
	 */
	public boolean occupe(Case c);
	
}