package rushhour.abstraction;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Optional;

public class Plateau implements IPlateau{
	// VARIABLES D'INSTANCE
	private int largeur ; // le nombre de colonnes du plateau
	private int hauteur ; // le nombre de ligne du plateau
	private Case sortie ; // la case correspondant a la sortie
	private List<IVehicule> vehicules ; // Les vehicules figurant sur le plateau

	/**
	 * Initialise le plateau this de sorte qu'il comporte
	 * largeur colonnes, hauteur lignes, une sortie a l'intersection de la colonne d'index
	 * colSortie et de la ligne d'index ligSortie, et les vehicules de la liste vehicules.
	 * @param largeur
	 * @param hauteur
	 * @param colSortie
	 * @param ligSortie
	 * @param vehicules
	 */
	public Plateau(int largeur, int hauteur, Case sortie, List<IVehicule> vehicules) {
		this.largeur = largeur;
		this.hauteur = hauteur;
		this.sortie = sortie;
		this.vehicules = vehicules;
		if (this.vehicules==null) {
			System.out.println("VOUS DEVEZ FOURNIR AU CONSTRUCTEUR DE PLATEAU UNE LIST DE VEHICULES CREE A PARTIR DES DONNEES DE LA CARTE");
			this.vehicules=new ArrayList<IVehicule>();
		}
	}
	/**
	 * Retourne le nombre de colonnes du plateau
	 * @return le nombre de colonnes du plateau
	 */
	public int getLargeur() {
		return this.largeur;
	}
	/**
	 *  Retourne le nombre de lignes du plateau
	 * @return le nombre de lignes du plateau
	 */
	public int getHauteur() {
		return this.hauteur;
	}
	/**
	 *  Retourne la case correspondant a la sortie
	 * @return Retourne la case correspondant a la sortie
	 */
	public Case getSortie() {
		return this.sortie;
	}
	/**
	 * Retourne une liste de IVehicule contenant l'ensemble des vehicules figurant sur le plateau
	 * @return une liste de IVehicule contenant l'ensemble des vehicules figurant sur le plateau
	 */
	public List<IVehicule> getVehicules() {
		return this.vehicules;
	}

	/**
	 * Retourne true si la case c est situee dans les limites du plateau
	 * @param c, une case quelconque
	 * @return true si la case c est situee dans les limites du plateau. Sinon, retourne false.
	 */
	public boolean contient(Case c) {
		return c.getCol()>=0 && c.getCol()<this.getLargeur() && c.getLig()>=0 && c.getLig()<this.getHauteur();
	}
	/**
	 * Retourne true si le vehicule rouge (le premier dans la liste de vehicule) occupe la case de la sortie.
	 * Sinon, retourne false.
	 * @return Retourne true si le vehicule rouge (le premier dans la liste de vehicule) occupe la case de la sortie. Sinon, retourne false.
	 */
	public boolean solutionTrouvee() {
		IVehicule v =this.getVehicules().get(0);
		return v.occupe(this.getSortie());
	}
	/**
	 *  Retourne le IVehicule qui qui occupe la case c (retourne null si aucun vehicule n'occupe la case c).
	 * @param c, une case de l'aire de jeu
	 * @return  le IVehicule qui qui occupe la case c (retourne null si aucun vehicule n'occupe la case c).
	 */
	public IVehicule vehiculeEn(Case c) {
		for(IVehicule v : this.vehicules)
					if (v.getCases().contains(c)) return v;
				return null;
		// ou
		/*
		IVehicule res = null;
		Iterator<IVehicule> i = this.getVehicules().iterator();
		while (res==null && i.hasNext()) {
			IVehicule v = i.next();
			if (v.occupe(c)){
				res = v;
			}
		}
		return res; */
//		// ou
//		/*
//		IVehicule res = null;
//		int pos=0;
//		while (res==null && pos<this.getVehicules().size()) {
//			if ((this.getVehicules().get(pos).occupe(c))){
//				res = this.getVehicules().get(pos);
//			}
//			pos++;
//		}
//		return res;
//		*/
//	    // OU --- mais c'est hors programme
//       Optional<IVehicule> vehicule = vehicules.stream().filter(v -> v.occupe(c)).findFirst();
//       return vehicule.orElse(null);
	}
}
