package rushhour.abstraction;

import java.util.ArrayList;
import java.util.List;

public class Vehicule implements IVehicule {
	// VARIABLES D'INSTABCES
	private int longueur; // le nombre de cases occupees par le vehicule
	private boolean orientation; // vaut IVehicule.HORIZONTALE ou IVehicule.VERTICALE
	private Case hautGauche; // la case la plus en haut a gauche occupee par le vehicule
	private int couleur;  //  la couleur exprimee au travers de la teinte.
	private String inscription; // l'inscription figurant sur le vehicule   

	/**
	 * Initialise this de sorte que :
	 * longueur soit le nombre de cases occupees par le vehicule designe par this, 
	 * orientation soit son orientation (Constante IVehicule.HORIZONTALE ou IVehicule.VERTICALE), 
	 * hautGauche soit la case la plus en haut a gauche qu'il occupe, 
	 * couleur soit l'entier designant sa teinte,
	 * et que inscription soit son inscription
	 * @param longueur
	 * @param orientation
	 * @param hautGauche
	 * @param couleur
	 * @param inscription
	 */
	public Vehicule(int longueur, boolean orientation, Case hautGauche, int couleur, String inscription) {
		this.longueur = longueur;
		this.orientation = orientation;
		this.hautGauche = hautGauche ; 
		this.couleur = couleur;
		this.inscription = inscription; 
	}
	/**
	 *  Retourne le nombre de cases occupees par le vehicule
	 * @return le nombre de cases occupees par le vehicule
	 */
	public int getLongueur() {
		return this.longueur;
	}
	/**
	 * Retourne l'orientation du vehicule (HORIZONTALE ou VERTICALE)
	 * @return l'orientation du vehicule (HORIZONTALE ou VERTICALE)
	 */
	public boolean getOrientation() {
		return this.orientation;
	}
	/**
	 * Retourne la case la plus en haut a gauche occupee par le vehicule
	 * @return la case la plus en haut a gauche occupee par le vehicule
	 */
	private Case getHautGauche() {
		return this.hautGauche;
	}
	/**
	 * Retourne la liste des cases occupees par le vehicule en allant de la case la plus en haut a gauche occupee a la case la plus en bas a droite occupee. 
	 * @return la liste des cases occupees par le vehicule en allant de la case la plus en haut a gauche occupee a la case la plus en bas a droite occupee.
	 */
	public List<Case> getCases() {
		List<Case> res = new ArrayList<Case>();
		if (this.getOrientation()==IVehicule.HORIZONTALE) {
			for (int col = this.getHautGauche().getCol(); col<this.getHautGauche().getCol()+this.getLongueur(); col++) {
				res.add(new Case(col, this.getHautGauche().getLig()));
			}
		} else {
			for (int lig = this.getHautGauche().getLig(); lig<this.getHautGauche().getLig()+this.getLongueur(); lig++) {
				res.add(new Case(this.getHautGauche().getCol(), lig));
			}
		}
		return res;
	}

	/**
	 * Retourne la teinte du vehicule
	 * @return la teinte du vehicule
	 */
	public int getCouleur() {
		return this.couleur;
	}
	/**
	 * Retourne l'inscription du vehicule
	 * @return l'inscription du vehicule
	 */
	public String getInscription() {
		return this.inscription;
	}


	/**
	 * Modifie le vehicule de sorte que la case la plus en haut a gauche qu'il occupe soit c
	 * Les autres caracteristiques (orientation, longueur, couleur et description) ne sont pas modifiees.
	 * @param c, une case de l'aire de jeu
	 */
	public void deplacerEn(Case c) {
		this.hautGauche = c;
	}

	/**
	 * Retourne true si le vehicule occupe (entre autres) la case c.
	 * @param c, une case de l'aire de jeu
	 * @return true si le vehicule occupe (entre autres) la case c.
	 */
	public boolean occupe(Case c) {
		return this.getCases().contains(c);
		// ou
		/*
		return ((this.getOrientation()==IVehicule.HORIZONTALE)
				&& (this.getHautGauche().getLig()==c.getLig()) 
				&& (this.getHautGauche().getCol()<=c.getCol()) 
				&& (this.getHautGauche().getCol()+this.getLongueur()-1>=c.getCol())) 
		||((this.getOrientation()==IVehicule.VERTICALE)
				&& (this.getHautGauche().getCol()==c.getCol()) 
				&& (this.getHautGauche().getLig()<=c.getLig()) 
				&& (this.getHautGauche().getLig()+this.getLongueur()-1>=c.getLig())); 
		 */
	}
}
